﻿using PdfSharp;
using PdfSharp.Pdf;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Diagnostics;
using System.IO;
using System.Text;
using System.Web.Mvc;
using ThinkGeo.MapSuite;
using ThinkGeo.MapSuite.Drawing;
using ThinkGeo.MapSuite.Layers;
using ThinkGeo.MapSuite.Mvc;
using ThinkGeo.MapSuite.Shapes;
using ThinkGeo.MapSuite.Styles;

namespace MVCApp.Controllers
{
    public class HomeController : Controller
    {
        private static readonly string baseDirectory;

        static HomeController()
        {
            baseDirectory = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "App_Data");
        }

        // GET: Home
        public ActionResult Index()
        {
            Map map = new Map("Map1",
               new System.Web.UI.WebControls.Unit(800, System.Web.UI.WebControls.UnitType.Pixel),
               new System.Web.UI.WebControls.Unit(600, System.Web.UI.WebControls.UnitType.Pixel));

            map.MapUnit = GeographyUnit.DecimalDegree;

            LayerOverlay layerOverlay = new LayerOverlay("layerOverlay");
            map.CustomOverlays.Add(layerOverlay);

            ShapeFileFeatureLayer layer = new ShapeFileFeatureLayer(Request.MapPath("~/App_Data/cities_a.shp"));
            layer.Name = "cities";
            layer.ZoomLevelSet.ZoomLevel01.DefaultTextStyle = TextStyles.CreateSimpleTextStyle("AREANAME", "Roboto", 7, DrawingFontStyles.Regular, GeoColor.SimpleColors.Black);
            layer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;
            layer.Encoding = Encoding.GetEncoding(1256);
            layerOverlay.Layers.Add(layer);

            layer.Open();
            map.CurrentExtent = layer.GetBoundingBox();
            return View(map);
        }

        [MapActionFilter]
        public void ToPdf(Map map, GeoCollection<object> args)
        {
            PdfGeoCanvas pdfGeoCanvas = new PdfGeoCanvas();

            PdfDocument document = new PdfDocument();

            PdfPage page = document.AddPage();

            page.Orientation = PageOrientation.Portrait;

            page.Size = PageSize.A4;

            LayerOverlay layerOverlay = (LayerOverlay)map.CustomOverlays["layerOverlay"];
            RectangleShape printExtent = layerOverlay.GetBoundingBox();
            foreach (Layer layer in layerOverlay.Layers)
            {
                pdfGeoCanvas.BeginDrawing(page, printExtent, map.MapUnit);
                layer.Open();
                layer.Draw(pdfGeoCanvas, new Collection<SimpleCandidate>());
                layer.Close();
                pdfGeoCanvas.EndDrawing();
            }

            string printingInfoFilePath = Path.Combine(baseDirectory, "Print Result.pdf");
            document.Save(printingInfoFilePath);
            Process.Start(printingInfoFilePath);
        }
    }
}